﻿using Microsoft.Extensions.Logging;
using ShipyardContracts.BindingModels;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.SearchModels;
using ShipyardDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ShipyardView
{
    public partial class FormShip : Form
    {
        private readonly ILogger _logger;
        private readonly IShipLogic _logic;
        private int? _id;
        private Dictionary<int, (IDetailModel, int)> _shipDetails;
        public int Id { set { _id = value; } }
        public FormShip(ILogger<FormShip> logger, IShipLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
            _shipDetails = new Dictionary<int, (IDetailModel, int)>();
        }

        private void FormShip_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка корабля");
                try
                {
                    var view = _logic.ReadElement(new ShipSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxName.Text = view.ShipName;
                        textBoxPrice.Text = view.Price.ToString();
                        _shipDetails = view.ShipDetails ?? new Dictionary<int, (IDetailModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки корабля");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
                }
            }

        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка деталей корабля");
            try
            {
                if (_shipDetails != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var pc in _shipDetails)
                    {
                        dataGridView.Rows.Add(new object[] { pc.Key,pc.Value.Item1.DetailName, pc.Value.Item2 });
                    }
                    textBoxPrice.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки компонент корабля");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormShipDetail));
            if (service is FormShipDetail form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.DetailModel == null)
                    {
                        return;
                    }
                    _logger.LogInformation("Добавление нового детали:{ DetailName} - { Count}", form.DetailModel.DetailName, form.Count);
                    if (_shipDetails.ContainsKey(form.Id))
                    {
                        _shipDetails[form.Id] = (form.DetailModel,form.Count);
                    }
                    else
                    {
                        _shipDetails.Add(form.Id, (form.DetailModel,form.Count));
                    }
                    LoadData();
                }
            }
        }

        private void ButtonUpd_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormShipDetail));
                if (service is FormShipDetail form)
                {
                    int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value);
                    form.Id = id;
                    form.Count = _shipDetails[id].Item2;
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.DetailModel == null)
                        {
                            return;
                        }
                        _logger.LogInformation("Изменение детали: { DetailName} - { Count}", form.DetailModel.DetailName, form.Count);
                        _shipDetails[form.Id] = (form.DetailModel,form.Count);
                        LoadData();
                    }
                }
            }
        }

        private void ButtonDel_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос",
               MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление детали:{ DetailName} - { Count}", dataGridView.SelectedRows[0].Cells[1].Value);
                        _shipDetails?.Remove(Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка",MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    LoadData();
                }
            }
        }
        private void ButtonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка",MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxPrice.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
                return;
            }
            if (_shipDetails == null || _shipDetails.Count == 0)
            {
                MessageBox.Show("Заполните компоненты", "Ошибка",MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение корабля");
            try
            {
                var model = new ShipBindingModel
                {
                    Id = _id ?? 0,
                    ShipName = textBoxName.Text,
                    Price = Convert.ToDouble(textBoxPrice.Text),
                    ShipDetails = _shipDetails
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение",MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения корабля");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
        private double CalcPrice()
        {
            double price = 0;
            foreach (var elem in _shipDetails)
            {
                price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
            }
            return Math.Round(price * 1.1, 2);
        }
    }
}
